'' =================================================================================================
''
''   File....... multiply_test.spin
''   Purpose.... template file for PASM subroutines
''   Author..... Jon "JonnyMac" McPhalen
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 
''
'' =================================================================================================


con

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal
' _xinfreq = 6_250_000                                          ' use 6.25MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq
  MS_001   = CLK_FREQ / 1_000
  US_001   = CLK_FREQ / 1_000_000


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con 

  #(-1), IS_ON, IS_OFF

  #0, LSBFIRST, MSBFIRST
  

con

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


obj

  term : "fullduplexserial64"
  rr   : "realrandom"                                           ' real random from hardware
  prng : "efx_prng"                                             ' pseudo-random number generator
  

var

  long  p1
  long  p2
  long  rspin
  long  rpasm


dat

Header                  byte    "Multiplication Test", CR
                        byte    CR
                        byte    "Param1       Param2       Spin         PASM       ", CR
                        byte    "-----------  -----------  -----------  -----------", CR
                        byte    0

Error                   byte    " !", 0


pub main | x

  term.start(RX1, TX1, %0000, 115_200)                          ' start terminal

  start                                                         ' start pasm cog

  rr.start 
  prng.seed(rr.random, rr.random, rr.random, rr.random, rr.random)  
  rr.stop   

  term.tx(CLS)
  term.str(@Header)

  repeat
    p1 := prng.random                                           ' randomize values
    p2 := prng.random
    rspin := p1 * p2                                            ' get results
    rpasm := command1(p1, p2)
                                                                
    repeat x from 0 to 3                                        ' display everything  
      term.tx(GOTOX)
      term.tx(lookupz(x: 0, 13, 26, 39))
      term.rjdec(long[@p1][x], 11, " ")

    if (rpasm <> rspin)                                         ' match?
      term.str(@Error)
      quit
    else
      term.tx(CR)
      pause(100)


  ' stop on error
      
  repeat 
    waitcnt(0)
    

con

  { ------------- }
  {  B A S I C S  } 
  { ------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1792                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

con

  { ----------------------- }
  {  O B J E C T   C O D E  } 
  { ----------------------- }

  { Remove everything above this section after object code has been vetted }
  

var

  long  cog

  long  cmd                                                     ' command
  long  param1                                                  ' parameter 1 / result
  long  param2                                                  ' parameter 2
  
 
pub start  

  stop

  ' set cog parameters

  param1 := clkfreq / 1_000                                     ' set timing "ticks"
  param2 := clkfreq / 1_000_000
  
  cog := cognew(@entry, @cmd) + 1                               ' start the cog

  return cog


pub stop

'' Stops cog (if running)

  if (cog)
    cogstop(cog - 1)
    cog := 0


pub command1(x, y)                                              ' test multiplication

  longmove(@param1, @x, 2)                                      ' copy parameters                          
  cmd := 1                                                      ' set command
  repeat while cmd                                              ' wait for completion

  return param1                                                 ' return result


pub command2

  cmd := 2
  repeat while cmd

  return param1


pub command3

  cmd := 3
  repeat while cmd

  return param1  


dat

                        org     0

entry                   mov     t1, par                         ' copy hub address of cmd
                        add     t1, #4                          ' hub address of param1
                        mov     p1pntr, t1                      ' save
                        rdlong  ms001, p1pntr                   ' read ticks per millisecond
                        add     t1, #4                          ' hub address of param2
                        mov     p2pntr, t1                      ' save
                        rdlong  us001, p2pntr                   ' read ticks per microsecond
                        jmp     #cmdexit                        ' clear command to start
                        
                                                                                                                 
getcmd                  rdlong  t1, par                 wz      ' check for command
        if_z            jmp     #getcmd
        
                        min     t1, #0                          ' limit range
                        max     t1, #4                          ' commands + 1
                        add     t1, #jmptable                   ' prep for jump
                        jmp     t1                              ' do it
                                                                 
jmptable                jmp     #cmdexit                         
do_cmd1                 jmp     #cmd1                            
do_cmd2                 jmp     #cmd2                            
do_cmd3                 jmp     #cmd3                            
                                                                 
cmdexit                 mov     t1, #0                          ' clear old command
                        wrlong  t1, par
                        jmp     #getcmd


                        
' handler from cmd == 1

cmd1                    rdlong  mx, p1pntr
                        add     t1, #4
                        rdlong  my, p2pntr
                        call    #mult32
                        wrlong  mresult, p1pntr
                        jmp     #cmdexit



' handler from cmd == 2

cmd2                    ' code here
                        ' -- write result to p1pntr

                        jmp     #cmdexit



' handler from cmd == 3

cmd3                    ' code here
                        ' -- write result to p1pntr

                        jmp     #cmdexit                                               
                              


' delay in milliseconds
' -- pass timing in "delay"

msdelay                 mov     timer, cnt                      ' sync timer
                        add     timer, ms001                    ' set for 1ms
                        sub     timer, #28                      ' account for setup/call overhead 
:loop                   waitcnt timer, ms001                    ' wait, reset
                        djnz    delay, #:loop
msdelay_ret             ret



' delay in microseconds
' -- pass timing in "delay"

usdelay                 mov     timer, cnt                      ' sync timer
                        add     timer, us001                    ' set for 1us
                        sub     timer, #28                      ' account for setup/call overhead 
:loop                   waitcnt timer, us001                    ' wait, reset
                        djnz    delay, #:loop
usdelay_ret             ret



' read element from PASM array
' -- pass table address in "tpntr"
' -- pass element index in "tidx"
' -- result returned in "tvalue"

read                    mov     t1, tpntr
                        add     t1, tidx
                        movs    rdval, t1                       ' set source for mov
                        nop
rdval                   mov     tvalue, 0-0
read_ret                ret



' write element to PASM array
' -- pass table address in "tpntr"
' -- pass element index in "tidx"
' -- pass value to write in "tvalue"

write                   mov     t1, tpntr
                        add     t1, tidx
                        movd    wrval, t1                       ' set destination for mov
                        nop
wrval                   mov     0-0, tvalue
write_ret               ret   



' signed, 32-bit multiply
' -- by "denis_potatoe" in Propeller forum (converted from C source)
' -- pass values in "mx" and "my"
' -- result returned in "mresult"

mult32                  mov     mresult, #0                     ' clear result

                        ' move "smallest" value to mx

                        abs     mx1, mx
                        abs     my1, my                        
                        cmp     mx1, my1                wc, wz
        if_nc           mov     my1, my                         ' swap mx and my
        if_nc           mov     my, mx
        if_nc           mov     mx, my1

                        ' multiply

                        test    mx, #1                  wz
        if_nz           sub     mresult, my
                        shl     my, #1
:loop                   test    mx, #1                  wc
                        test    mx, #2                  wz
       if_nz_and_nc     sub     mresult, my                 
       if_z_and_c       add     mresult, my                 
                        shl     my, #1                  wz
       if_nz            sar     mx, #1                  wz
       if_nz            cmp     mx, NEG_ONE             wz
       if_nz            jmp     #:loop

mult32_ret              ret

' --------------------------------------------------------------------------------------------------

NEG_ONE                 long    -1

p1pntr                  res     1                               ' hub address of param1
p2pntr                  res     1                               ' hub address of param2

ms001                   res     1                               ' clock ticks per millisecond
us001                   res     1                               ' clock ticks per microsecond
delay                   res     1                               ' units to delay
timer                   res     1                               ' for timing

tpntr                   res     1                               ' cog address of table
tidx                    res     1                               ' table index
tvalue                  res     1                               ' table value for r/w

mresult                 res     1                               ' for multiplication
mx                      res     1
my                      res     1
mx1                     res     1
my1                     res     1

t1                      res     1                               ' work vars
t2                      res     1

                        fit     496                                    

                        
dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}  